{*********************************************************}
{* File       : DBF_LIB.INC                              *}
{* Vers.      : 1.01                                     *}
{* Last Edit  : 19.4.94                                  *}
{* Autor      : G. Born                                  *}
{* Progr. Spr.: Turbo Pascal 4.0/5.0/6.0/7.0             *}
{* Funktion   : Library mit den Funktionen zum Zugriff   *}
{*              aus Turbo Pascal auf dBASE-DBF-Files     *}
{*********************************************************}

procedure USE (Var handle : file; Filename: Name; Var status: integer);
{--------------------------------------------------------}
{ Die Routine ffnet die angegebene Datei, liest den     }
{ Dateiheader und prft, ob es sich um eine gltige      }
{ DBASE III-DBF-Datei handelt. Fr die Parameter gilt:   }
{                                                        }
{    handle    Filehandle der geffneten Datei           }
{    Filename  DOS-Filename der DBF-Datei                }
{    status    Status des Aufrufes                       }
{               0 -> ok                                  }
{               1 -> Fehler: keine Datei gefunden        }
{               2 -> Fehler: keine DBF-Datei (dBASE III) }
{               3 -> Fehler: dBASE II DBF-Datei          }
{               4 -> Fehler: EOF erreicht                }
{               5 -> Fehler: kein Header Ende gefunden   }
{               6 -> Fehler: Feldbeschreibung falsch     }
{                                                        }
{ ber handle lassen sich mehrere DBF-Files verwalten.   }
{--------------------------------------------------------}
var i,j : integer;
    headend: byte;

begin

{--------------------------------------------------------}
{  ffne die Datei und prfe das Ergebnis des Aufrufes   }
{--------------------------------------------------------}

  status := 0;                         {* clear status flag  *}
  Assign(handle,Filename);             {* setze Filename     *}
  {$I-}                                {* Errorcheck off     *}
  Reset(handle);                       {* open file          *}
  {$I+}                                {* Errorcheck on      *}
  if IOResult <> 0 then
   begin
    status := 1;                       {* Fehlerexit         *}
    exit;
   end;

{--------------------------------------------------------}
{  Lese den Kopf der Datei und prfe die Signatur        }
{--------------------------------------------------------}

 BlockRead (handle, kopf,1);           {* lese Kopf der Datei    *}

 IF EOF(handle) THEN                   {* DBASE Datei ?          *}
 begin
  status := 4;                         {* Fehlerexit             *}
  exit;
 end;

 With kopf do                          {* prfe Version          *}
  begin
   IF (version <> $83) AND
    (version <> $03) THEN              {* kein DBASE Header ?    *}
    IF version = $02 THEN
     begin
      status := 3;                     {* DBASE II Header        *}
      exit;
     end
    ELSE
     begin
      status := 2;                     {* kein DBF-Header        *}
      exit;
     end;
  end;

{--------------------------------------------------------}
{  lese und decodiere die Feldbescheibung der DBASE III- }
{  Datei, es sind maximal 128 Felder zulssig            }
{--------------------------------------------------------}

 anzahl := ((kopf.headerb - 1) div 32) - 1; {* Zahl der Felder     *}

{* Reset Datei um mit neuer Puffersize zugreifen zu knnen         *}

 Reset (handle,32); 
 SEEK (handle, 1);                     {* setze Zeiger auf 1. Feld *}
 FOR i := 1 TO anzahl do               {* lese n Felddefinitionen  *}
 begin
  BlockRead (handle,felder[i],1);      {* lese Definition Feld     *}
  IF EOF(handle) THEN                  {* Fehler abfangen?         *}
   begin
    status := 6;
    exit;
   end;
 end;

{--------------------------------------------------------}
{ prfe ob nchstes Byte das Header Ende signalisiert    }
{--------------------------------------------------------}
 reset (handle,1);                     {* als File mit Bytes     *}
 seek (handle,kopf.headerb-1);         {* auf Endezchn i. Kopf   *}
 BlockRead (handle,headend,1);         {* lese Endezeichen       *}
 IF headend <> $0D THEN                {* Ende = 0DH             *}
  begin
    status := 5;
    exit;
  end;

  recnr := 1;                          {* auf 1. Datensatz       *}
  reset (handle,1);               {* Datei als Bytestream ffnen *}

end; {***** USE ******}

procedure GetRecord (var handle: file; var status: integer; var satz : dbsatz);
{--------------------------------------------------------}
{  lese einen Satz aus der DBASE III - Datenbank und     }
{  gebe das Ergebnis in satz zurck. Die Daten sind      }
{  als ASCII - Text in der Datenbank abgelegt.           }
{--------------------------------------------------------}

Var recptr : longint;                  {* Recordnummer           *}

begin
 status := 0;                          {* clear status            *}
 recptr := kopf.headerb + ((recnr-1) * kopf.recordb); {* Satzptr. *}
 seek (handle,recptr);                 {* positioniere Lesezeiger *}
 BlockRead (handle, satz,kopf.recordb);{* lese Satz in Buffer     *}
 IF EOF(handle) THEN                   {* EOF erreicht -> Fehler  *}
  begin
   status := 4;
   exit;
  end;
end;  {******* GetRecord *****}

procedure PutRecord (var handle: file; var status: integer; var satz : dbsatz);
{--------------------------------------------------------}
{  schreibe einen Satz in die DBASE III - Datenbank.     }
{  Die Daten sind als ASCII - Text im Puffer, geordnet   }
{  nach Feldern, abzulegen.                              }
{--------------------------------------------------------}
Var jahr, monat, tag, wtag : word;     {* Datum                    *}
    recptr : longint;
begin
 status := 0;
 recptr := kopf.headerb + ((recnr-1) * kopf.recordb); {* Satzptr. *}
 seek (handle,recptr);                 {* positioniere Schreibzeiger *}
 BlockWrite(handle,satz,kopf.recordb); {* in die Datenbank schreiben *}

 GetDate (jahr,monat,tag,wtag);        {* lese Datum                 *}

 kopf.datum[1] := jahr MOD 100;        {* Jahr 0 .. 99               *}
 kopf.datum[2] := monat;               {* Monat                      *}
 kopf.datum[3] := tag;                 {* Tag                        *}

 seek (handle,1);                      {* auf Datumsfeld             *}
 BlockWrite (handle,kopf.datum,3);     {* Datum aktualisieren        *}

end; { ****** PutRecord ***** }

procedure AppendBlank (var handle : file; var status : integer);
{--------------------------------------------------------}
{ Hnge einen leeren Satz in die DBASE III - Datenbank an}
{ nach dem Aufruf steht der Schreiblesezeiger auf diesem }
{ Satz, d.h. PutRecord kann direkt Daten speichern.      }
{--------------------------------------------------------}
Var tmp : byte;
    i   : Integer;
    jahr, monat, tag, wtag : word;     {* Datum                    *}
    recptr : longint;
begin

 status := 0;
 recnr := kopf.records + 1;            {* calc record nr            *}
 recptr := kopf.headerb + ((recnr-1) * kopf.recordb); {* Satzptr. *}
 seek (handle,recptr);                 {* auf Ende                  *}
 For i := 1 to kopf.recordb do
  begin
   tmp := $20;                         {* setz Blank in Puffer und   *}
   BlockWrite (handle,tmp,1);          {* in Datei schreiben         *}
  end;
 tmp := $1A;                           {* EOF - Marke anhngen       *}
 BlockWrite (handle,tmp,1);            {* schreibe Marke             *}

 GetDate (jahr,monat,tag,wtag);        {* lese Datum                 *}

 kopf.datum[1] := jahr MOD 100;        {* Jahr 0 .. 99               *}
 kopf.datum[2] := monat;               {* Monat                      *}
 kopf.datum[3] := tag;                 {* Tag                        *}

 seek (handle,1);                      {* auf Datumsfeld             *}
 BlockWrite (handle,kopf.datum,3);     {* Datum aktualisieren        *}

 kopf.records := kopf.records + 1;     {* Korrektur Recordzahl       *}
 seek (handle, 4);                     {* auf Position in Header     *}
 BlockWrite (handle,kopf.records,4);   {* Header aktualisieren       *}

end; { ***** AppendBlank ***** }

procedure Skip(n: integer; var status : integer);
{----------------------------------------------------------}
{ Positioniere den Schreib- / Lesezeiger um n Stze weiter.}
{ status = 0 -> ok                                         }
{          1 -> unteres Limit erreicht                     }
{          2 -> oberes Limit erreicht                      }
{----------------------------------------------------------}

begin
 status := 0;
 recnr := recnr + n;                   {* nchster Satz              *}
 If recnr < 1 then
  begin
   recnr := 1;                         {* unteres Limit              *}
   status := 1;                        {* Fehlerexit                 *}
   exit;
  end;

 If recnr > kopf.records then
  begin
   recnr := kopf.records;              {* oberes  Limit              *}
   status := 2;                        {* Fehlerexit                 *}
   exit;
  end;

end; { ****** Skip ***** }

procedure GotoTop;
{----------------------------------------------------------}
{ Positioniere den Schreib- / Lesezeiger auf Satz 1.       }
{----------------------------------------------------------}
begin
 recnr := 1;                           {* erster Satz                *}
end; { ****** GotoBottom ***** }

procedure GotoBottom;
{----------------------------------------------------------}
{ Positioniere den Schreib- / Lesezeiger auf letzten Satz  }
{----------------------------------------------------------}
begin
 recnr := kopf.records;                {* letzter Satz               *}
end; { ****** GotoTop ***** }

function DBEof : boolean;
{----------------------------------------------------------}
{ Prfe, ob EOF() der Datenbank erreicht ist               }
{----------------------------------------------------------}
begin                                  {* Dateiende erreicht ?       *}
 IF recnr >= kopf.records THEN
  DBEof := true                        {* ja                         *}
 ELSE
  DBEof := false;                      {* nein                       *}

end;  { ****** DBEof ***** }

{*  Ende der Libdatei   *}

